import gdsfactory as gf
from gdsfactory.component import Component
from gdsfactory.components.taper_cross_section import taper_cross_section
from gdsfactory.cross_section import (
    CrossSection,
    LayerSpec,
    Section,
    cross_section,
)

from gvtt.xsections import rib, strip


def xs_rib_strip(
    width: float = 3.0,
    width_trench: float = 10.0,
    width_deep: float = 10.75,
    dist_deep: float = 3.0,
    wg_marking_layer: LayerSpec | None = None,
    **kwargs,
) -> CrossSection:
    """Return CrossSection of strip waveguide defined by trenches."""

    sections = kwargs.pop("sections", [])
    sections += [
        Section(width=width, layer="WG_RIBS_SUB", name="WG", simplify=None),
        Section(
            width=2.0 * width_trench + width,
            layer="WG_RIBS_ADD",
            name="trench",
            simplify=None,
        ),
        Section(
            width=width + 2.0 * (dist_deep + width_deep),
            layer="WG_MOST_ADD",
            name="trench_deep",
            simplify=None,
        ),
        Section(
            width=width + 2.0 * dist_deep,
            layer="WG_MOST_SUB",
            name="trench_cut",
            simplify=None,
        ),
    ]

    return cross_section(
        width=width,
        layer=wg_marking_layer,
        sections=tuple(sections),
        port_names=("o1", "o2"),
        **kwargs,
    )


@gf.cell
def rib_to_strip(
    length=200.0,
    width1=3.0,
    width2=3.0,
) -> Component:
    """
    Standard rib-to-strip waveguide converter.
    """
    c = gf.Component()

    rib_strip_1 = xs_rib_strip(width=3.0, width_deep=0.5)  # rib end
    rib_strip_2 = xs_rib_strip(width=3.0, dist_deep=-0.25)  # strip end

    cn = c << gf.components.taper_cross_section(
        cross_section1=rib_strip_1,
        cross_section2=rib_strip_2,
        linear=True,
        length=length,
    )

    c.absorb(cn)
    c.info["length"] = length

    if width1 != 3.0:
        t1 = c << rib_taper(width1, 3.0)
        t1.connect("o2", cn.ports["o1"])
        c.add_port("o1", port=t1.ports["o1"])
        c.info["length"] += t1.info["length"]
        c.absorb(t1)
    else:
        c.add_port("o1", port=cn.ports["o1"])

    if width2 != 3.0:
        t2 = c << strip_taper(3.0, width2)
        t2.connect("o1", cn.ports["o2"])
        c.add_port("o2", port=t2.ports["o2"])
        c.info["length"] += t2.info["length"]
        c.absorb(t2)
    else:
        c.add_port("o2", port=cn.ports["o2"])

    c.info["width1"] = float(width1)
    c.info["width2"] = float(width2)

    return c


@gf.cell
def strip_taper(
    width1=1,
    width2=1,
    taper_ratio=25.0,
    length: float | None = None,
    **kwargs,
) -> Component:
    """
    Standard rib-to-strip waveguide converter.
    """
    length = length or abs(width1 - width2) * taper_ratio or 10e-3
    kwargs.pop("cross_section", None)
    c = gf.Component()

    ref = c << taper_cross_section(
        cross_section1=strip(width1),
        cross_section2=strip(width2),
        linear=True,
        npoints=2,
    )
    c.info["length"] = length
    c.info["width1"] = float(width1)
    c.info["width2"] = float(width2)
    c.add_ports(ref.ports)
    c.absorb(ref)
    return c


@gf.cell
def rib_taper(
    width1=1,
    width2=1,
    taper_ratio=50.0,
    length: float | None = None,
    **kwargs,
) -> Component:
    """
    Standard rib-to-strip waveguide converter.
    """
    length = length or abs(width1 - width2) * taper_ratio or 10e-3
    c = gf.Component()
    ref = c << taper_cross_section(
        cross_section1=rib(width1),
        cross_section2=rib(width2),
        length=length,
        linear=True,
        npoints=2,
        **kwargs,
    )
    c.info["length"] = length
    c.info["width1"] = float(width1)
    c.info["width2"] = float(width2)
    c.add_ports(ref.ports)
    c.absorb(ref)
    return c


if __name__ == "__main__":
    c = rib_taper()
    # c = strip_taper()
    # c.pprint_ports()
    c.show()



import gdsfactory as gf
from gdsfactory.component import Component

from gvtt.xsections import rib


@gf.cell
def edge_coupler_rib(
    edge_coupling_width: float = 3.0,
    polishing_length: float = 25.0,
    side: str = "W",
    xpos: float = 0.0,
    ypos: float = 0.0,
) -> Component:
    c = gf.Component()
    side = side.upper()

    if side == "W":
        orientation = 0
    if side == "E":
        orientation = 180
    if side == "N":
        orientation = -90
    if side == "S":
        orientation = 90

    ref = c << gf.components.straight(
        cross_section=rib(edge_coupling_width), length=polishing_length
    )
    ref.rotate(orientation)
    ref.move((xpos, ypos))

    c.add_port("o1", port=ref.ports["o2"])
    c.absorb(ref)
    return c


if __name__ == "__main__":
    c = edge_coupler_rib()
    c.show()



import gdsfactory as gf
from gdsfactory.components.text import text
from gdsfactory.typings import ComponentSpec, Float2, LayerSpec

import gvtt


def box(x0, y0, w, h):
    dw = w / 2.0
    dh = h / 2.0
    return [
        [x0 + dw, y0 + dh],
        [x0 + dw, y0 - dh],
        [x0 - dw, y0 - dh],
        [x0 - dw, y0 + dh],
    ]


@gf.cell
def die(
    size: tuple[float, float] = (5000.0, 9500.0),
    street_width: float = 75.0,
    street_length: float = 1000.0,
    die_name: str | None = "chip99",
    text_size: float = 100.0,
    text_location: str | Float2 = "SW",
    layer: LayerSpec | None = None,  # "FLOORPLAN",
    bbox_layer: LayerSpec | None = None,
    draw_corners: bool = True,
    draw_dicing_lane: bool = True,
    text_component: ComponentSpec = text,
) -> gf.Component:
    c = gf.Component(name="die")
    w, h = size[0], size[1]

    for layer, size in gvtt.frame_dimensions.items():
        c.add_polygon(box(w / 2.0 - size / 2.0, 0.0, size, h), layer=layer)
        c.add_polygon(box(-w / 2.0 + size / 2.0, 0.0, size, h), layer=layer)

        c.add_polygon(box(0.0, h / 2.0 - size / 2.0, w, size), layer=layer)
        c.add_polygon(box(0.0, -h / 2.0 + size / 2.0, w, size), layer=layer)

    if bbox_layer:
        c.add_polygon(
            [[w / 2, h / 2], [w / 2, -h / 2], [-w / 2, -h / 2], [-w / 2, h / 2]],
            layer=bbox_layer,
        )

    c.info["port_x_position_west"] = (
        -w / 2.0 + gvtt.frame_dimensions[gvtt.LAYER.WG_SNGL_ADD] + 1.5
    )
    c.info["port_x_position_east"] = (
        w / 2.0 - gvtt.frame_dimensions[gvtt.LAYER.WG_SNGL_ADD] - 1.5
    )

    c.info["port_y_position_north"] = (
        h / 2.0 - gvtt.frame_dimensions[gvtt.LAYER.WG_SNGL_ADD] - 1.5
    )
    c.info["port_y_position_south"] = (
        -h / 2.0 + gvtt.frame_dimensions[gvtt.LAYER.WG_SNGL_ADD] + 1.5
    )

    return c


if __name__ == "__main__":
    c = die()
    c.show()



from gdsfactory.technology import LayerMap
from gdsfactory.typings import Layer


class LayerMap(LayerMap):
    # waveguide types
    TYPE_RIB: Layer = (89, 0)
    TYPE_STRIP: Layer = (89, 1)

    AX_BBOX: Layer = (63, 10)
    AX_BPIN: Layer = (63, 12)
    AX_BVIZ: Layer = (63, 11)
    AX_INFO: Layer = (63, 63)
    AX_LINE: Layer = (63, 40)
    AX_NOME: Layer = (63, 1)
    AX_NOUS: Layer = (63, 9)
    AX_NOWG: Layer = (63, 2)
    AX_NOXX: Layer = (63, 0)
    AX_TRIP: Layer = (63, 41)
    AX_WARN: Layer = (63, 42)
    ME_ALUM_ADD: Layer = (2, 13)
    ME_ALUM_CPY: Layer = (2, 33)
    ME_ALUM_LF: Layer = (2, 3)
    ME_ALUM_SUB: Layer = (2, 23)
    ME_GOLD_ADD: Layer = (2, 12)
    ME_GOLD_CPY: Layer = (2, 32)
    ME_GOLD_LF_C: Layer = (2, 2)
    ME_GOLD_SUB: Layer = (2, 22)
    ME_HVIA_ADD: Layer = (2, 14)
    ME_HVIA_CPY: Layer = (2, 34)
    ME_HVIA_DF: Layer = (2, 4)
    ME_HVIA_SUB: Layer = (2, 24)
    ME_WIPO_ADD: Layer = (2, 16)
    ME_WIPO_CPY: Layer = (2, 36)
    ME_WIPO_DF_C: Layer = (2, 6)
    ME_WIPO_SUB: Layer = (2, 26)
    ME_WIRE_ADD: Layer = (2, 15)
    ME_WIRE_CPY: Layer = (2, 35)
    ME_WIRE_LF: Layer = (2, 5)
    ME_WIRE_SUB: Layer = (2, 25)
    NP_NIMP_ADD: Layer = (5, 13)
    NP_NIMP_CPY: Layer = (5, 33)
    NP_NIMP_DF: Layer = (5, 3)
    NP_NIMP_SUB: Layer = (5, 23)
    NP_PIMP_ADD: Layer = (5, 12)
    NP_PIMP_CPY: Layer = (5, 32)
    NP_PIMP_DF: Layer = (5, 2)
    NP_PIMP_SUB: Layer = (5, 22)
    WG_ARCO_ADD: Layer = (1, 14)
    WG_ARCO_CPY: Layer = (1, 34)
    WG_ARCO_LF: Layer = (1, 4)
    WG_ARCO_SUB: Layer = (1, 24)
    WG_DEOX_ADD: Layer = (1, 13)
    WG_DEOX_CPY: Layer = (1, 33)
    WG_DEOX_DF: Layer = (1, 3)
    WG_DEOX_SUB: Layer = (1, 23)
    WG_LANE_ADD: Layer = (1, 12)
    WG_LANE_CPY: Layer = (1, 32)
    WG_LANE_DF: Layer = (1, 2)
    WG_LANE_SUB: Layer = (1, 22)
    WG_MOST_ADD: Layer = (1, 11)
    WG_MOST_CPY: Layer = (1, 31)
    WG_MOST_DF: Layer = (1, 1)
    WG_MOST_SUB: Layer = (1, 21)
    WG_PEDE_ADD: Layer = (1, 16)
    WG_PEDE_CPY: Layer = (1, 36)
    WG_PEDE_LF: Layer = (1, 6)
    WG_PEDE_SUB: Layer = (1, 26)
    WG_RIBS_ADD: Layer = (1, 10)
    WG_RIBS_CPY: Layer = (1, 30)
    WG_RIBS_DF: Layer = (1, 0)
    WG_RIBS_SUB: Layer = (1, 20)
    WG_SNGL_ADD: Layer = (1, 15)
    WG_SNGL_CPY: Layer = (1, 35)
    WG_SNGL_DF: Layer = (1, 5)
    WG_SNGL_SUB: Layer = (1, 25)

    # AUX
    FLOORPLAN: Layer = (63, 98)

    # common gdsfactory layers
    LABEL_INSTANCE: Layer = (66, 0)
    DEVREC: Layer = (68, 0)
    PORT: Layer = (99, 10)
    PORTE: Layer = (99, 11)
    TE: Layer = (203, 0)
    TM: Layer = (204, 0)
    TEXT: Layer = (66, 0)


LAYER = LayerMap()


if __name__ == "__main__":
    import gdsfactory as gf
    from gdsfactory.technology.klayout_tech import KLayoutTechnology

    from gvtt.config import PATH

    # from gdsfactory.technology import lyp_to_dataclass

    # layers = lyp_to_dataclass(PATH.lyp)
    # print(layers)
    LAYER_VIEWS = gf.technology.LayerViews(filepath=PATH.lyp)
    LAYER_VIEWS.to_yaml(PATH.lyp_yaml)
    t = KLayoutTechnology(
        name="VTT",
        layer_map=dict(LAYER),
        layer_views=LAYER_VIEWS,
        # layer_stack=LAYER_STACK,
        # connectivity=connectivity,
    )
    t.write_tech(tech_dir=PATH.tech_dir)
    # print(LAYER)



from gvtt.components.bend_euler import (
    bend_euler,
)
from gvtt.components.die import (
    box,
    die,
)
from gvtt.components.edge_coupler_rib import (
    edge_coupler_rib,
)
from gvtt.components.mmi1x2 import (
    mmi1x2,
)
from gvtt.components.mmi2x2 import (
    mmi2x2,
)
from gvtt.components.transitions import (
    rib_taper,
    rib_to_strip,
    strip_taper,
    xs_rib_strip,
)

__all__ = [
    "bend_euler",
    "box",
    "die",
    "edge_coupler_rib",
    "mmi1x2",
    "mmi2x2",
    "rib_taper",
    "rib_to_strip",
    "strip_taper",
    "xs_rib_strip",
]



from functools import partial

from gdsfactory.components import mmi1x2 as _mmi1x2

from gvtt.components.transitions import strip_taper
from gvtt.xsections import xs_sc

mmi1x2 = partial(
    _mmi1x2,
    width_mmi=6.25,
    length_mmi=43.25,
    width_taper=1.875,
    cross_section=xs_sc,
    gap_mmi=(6.25 - 2 * 1.875) / 2,
    taper=strip_taper,
    length_taper=1.0,
)

if __name__ == "__main__":
    c = mmi1x2()
    c.show()



""" technology definitions."""
import sys

import gdsfactory as gf
from gdsfactory.cross_section import (
    CrossSection,
    LayerSpec,
    Section,
    cross_section,
    get_cross_sections,
)

from gvtt.layers import LAYER


def rib(
    width: float = 2.5,
    width_trench: float = 10.0,
    wg_marking_layer: LayerSpec = LAYER.TYPE_RIB,
    **kwargs,
) -> CrossSection:
    """Return CrossSection of rib waveguide defined by trenches.

    Args:
        width: waveguide width.
        width_trench: trench width.
        wg_marking_layer: layer for waveguide marking.
        kwargs: cross_section arguments.
    """

    sections = kwargs.pop("sections", [])
    sections += [
        Section(width=width, layer="WG_RIBS_SUB", name="WG", simplify=None),
        Section(
            width=2.0 * width_trench + width,
            layer="WG_RIBS_ADD",
            name="trench",
            simplify=None,
        ),
    ]

    return cross_section(
        width=width,
        layer=wg_marking_layer,
        sections=tuple(sections),
        port_names=("o1", "o2"),
        **kwargs,
    )


def strip(
    width: float = 1.875,
    width_trench: float = 10.0,
    wg_marking_layer: LayerSpec = LAYER.TYPE_STRIP,
    **kwargs,
) -> CrossSection:
    """Return CrossSection of strip waveguide defined by trenches.

    Args:
        width: waveguide width.
        width_trench: trench width.
        wg_marking_layer: layer for waveguide marking.
        kwargs: cross_section arguments.
    """

    sections = kwargs.pop("sections", [])
    sections += [
        Section(width=width, layer="WG_RIBS_SUB", name="WG", simplify=None),
        Section(
            width=2.0 * width_trench + width,
            layer="WG_RIBS_ADD",
            name="trench",
            simplify=None,
        ),
        Section(
            width=2.0 * width_trench + width + 1,
            layer="WG_MOST_ADD",
            name="trench_2",
            simplify=None,
        ),
    ]

    return cross_section(
        width=width,
        layer=wg_marking_layer,
        sections=tuple(sections),
        port_names=("o1", "o2"),
        **kwargs,
    )


def vttstrip(
    width: float = 1.875,
    width_trench: float = 10.0,
    wg_marking_layer: LayerSpec | None = LAYER.TYPE_STRIP,
    **kwargs,
) -> CrossSection:
    """Return CrossSection of strip waveguide defined by trenches."""

    sections = kwargs.pop("sections", [])
    sections += [
        Section(width=width, layer="WG_RIBS_SUB", name="WG", simplify=None),
        Section(
            width=2.0 * width_trench + width,
            layer="WG_RIBS_ADD",
            name="trench",
            simplify=None,
        ),
        Section(
            width=2.0 * width_trench + width + 1,
            layer="WG_MOST_ADD",
            name="trench_2",
            simplify=None,
        ),
    ]

    return cross_section(
        width=width,
        layer=wg_marking_layer,
        sections=tuple(sections),
        port_names=("o1", "o2"),
        **kwargs,
    )


sm_rib = rib(width=2.5)
xs_sc = euler_strip = strip(width=1.875)

cross_sections = get_cross_sections(sys.modules[__name__])

if __name__ == "__main__":
    print(cross_sections.keys())
    c = gf.components.straight(cross_section=rib)
    c.show()



from __future__ import annotations

import gdsfactory
from gdsfactory.config import PATH as GPATH
from gdsfactory.get_factories import get_cells
from gdsfactory.pdk import Pdk, constants

from gvtt import components
from gvtt.config import PATH
from gvtt.layers import LAYER
from gvtt.xsections import cross_sections

cells = get_cells([components])

frame_dimensions = {
    LAYER.WG_SNGL_ADD: 75.0,
    LAYER.WG_LANE_ADD: 71.5,
    LAYER.WG_DEOX_ADD: 85.0,
    LAYER.WG_ARCO_ADD: 100.0,
}

FRAME_LAYERS = {}

PORT_MARKER_LAYER_TO_TYPE = {
    LAYER.PORT: "optical",
    LAYER.PORTE: "dc",
    LAYER.TE: "vertical_te",
    LAYER.TM: "vertical_tm",
}

PORT_LAYER_TO_TYPE = {
    LAYER.TYPE_RIB: "optical",
    LAYER.TYPE_STRIP: "optical",
    LAYER.TE: "vertical_te",
    LAYER.TM: "vertical_tm",
}

PORT_TYPE_TO_MARKER_LAYER = {v: k for k, v in PORT_MARKER_LAYER_TO_TYPE.items()}

LAYER_TRANSITIONS = {
    LAYER.TYPE_RIB: "rib_taper",
    LAYER.TYPE_STRIP: "strip_taper",
    (LAYER.TYPE_RIB, LAYER.TYPE_STRIP): "rib_to_strip",
    (LAYER.TYPE_STRIP, LAYER.TYPE_RIB): "strip_to_rib",
}

LAYER_VIEWS = None  # LayerViews(filepath=PATH.klayout_yaml)

PDK = Pdk(
    name="VTT-3umSOI",
    cells=cells,
    cross_sections=cross_sections,
    layers=dict(LAYER),
    layer_stack=None,
    layer_views=LAYER_VIEWS,
    layer_transitions=LAYER_TRANSITIONS,
    constants=constants,
)

GPATH.sparameters = PATH.sparameters

# pdk.register_cells_yaml(dirpath=pathlib.Path(__file__).parent.absolute())

gdsfactory.routing.all_angle.LOW_LOSS_CROSS_SECTIONS = [
    {"cross_section": "rib", "settings": {"width": 2.5}},
    {"cross_section": "xs_sc", "settings": {"width": 6.0}},
    {"cross_section": "xs_sc", "settings": {"width": 3.0}},
]

PDK.activate()
__version__ = "0.0.3"

if __name__ == "__main__":
    # layer_views = LayerViews(filepath=PATH.lyp_yaml)
    # layer_views.to_lyp(PATH.lyp)

    print(PDK.name)



from __future__ import annotations

import gdsfactory as gf
import numpy as np
from gdsfactory.add_padding import get_padding_points
from gdsfactory.component import Component
from gdsfactory.components.wire import wire_corner
from gdsfactory.path import euler
from gdsfactory.typings import CrossSectionSpec, Optional


def _eulerR_1550(angle: float) -> float:
    if angle == 0:
        return 0.0
    p, v, a0 = 0.79, 2093.0, 18.75  # for 1550 nm, TE, 1.875 um
    return (v / max(abs(angle), a0)) ** (1.0 / p)


@gf.cell
def bend_euler(
    angle: float = 90.0,
    p: float = 1.0,
    with_arc_floorplan: bool = False,
    npoints: Optional[int] = None,
    direction: str = "ccw",
    with_bbox: bool = True,
    cross_section: CrossSectionSpec = "xs_sc",
    **kwargs,
) -> Component:
    """Returns an euler bend that transitions from straight to curved.

    By default, `radius` corresponds to the minimum radius of curvature of the bend.
    However, if `with_arc_floorplan` is True, `radius` corresponds to the effective
    radius of curvature (making the curve a drop-in replacement for an arc). If
    p < 1.0, will create a "partial euler" curve as described in Vogelbacher et.
    al. https://dx.doi.org/10.1364/oe.27.031394

    default p = 0.5 based on this paper
    https://www.osapublishing.org/oe/fulltext.cfm?uri=oe-25-8-9150&id=362937

    Args:
        angle: total angle of the curve.
        p: Proportion of the curve that is an Euler curve.
        with_arc_floorplan: If False: `radius` is the minimum radius of curvature
          If True: The curve scales such that the endpoints match a bend_circular
          with parameters `radius` and `angle`.
        npoints: Number of points used per 360 degrees.
        direction: cw (clock-wise) or ccw (counter clock-wise).
        with_bbox: add bbox_layers and bbox_offsets to avoid DRC sharp edges.
        cross_section: specification (CrossSection, string, CrossSectionFactory dict).
        kwargs: cross_section settings.

    .. code::

                  o2
                  |
                 /
                /
               /
       o1_____/
    """

    x = gf.get_cross_section(cross_section, **kwargs)
    radius = _eulerR_1550(abs(angle))

    if radius is None:
        return wire_corner(cross_section=x)

    c = Component()

    p = euler(
        radius=radius, angle=angle, p=p, use_eff=with_arc_floorplan, npoints=npoints
    )

    ref = c << p.extrude(x)
    c.info["length"] = np.round(p.length(), 3)
    c.info["dy"] = np.round(abs(float(p.points[0][0] - p.points[-1][0])), 3)
    c.info["radius_min"] = np.round(p.info["Rmin"], 3)
    c.info["radius"] = p.xmax
    c.info["width"] = x.width

    if x.info:
        c.info.update(x.info)

    if with_bbox and x.bbox_layers:
        padding = []
        angle = int(angle)
        for offset in x.bbox_offsets:
            top = offset if angle in [180, -180, -90] else 0
            bottom = 0 if angle in [-90] else offset
            points = get_padding_points(
                component=c,
                default=0,
                bottom=bottom,
                right=offset,
                top=top,
            )
            padding.append(points)

        for layer, points in zip(x.bbox_layers, padding):
            c.add_polygon(points, layer=layer)

    if direction == "cw":
        ref.mirror(p1=[0, 0], p2=[1, 0])

    c.add_ports(ref.ports)
    c.absorb(ref)

    return c


if __name__ == "__main__":
    c = bend_euler()
    c.show(show_ports=True)



__all__ = ["PATH"]

import pathlib

home = pathlib.Path.home()
cwd = pathlib.Path.cwd()

cwd_config = cwd / "config.yml"

home_config = home / ".config" / "vtt3um.yml"
config_dir = home / ".config"
config_dir.mkdir(exist_ok=True)
module_path = pathlib.Path(__file__).parent.absolute()
repo_path = module_path.parent


class Path:
    layer_lib = module_path / "vtt-layer-list.yaml"
    module = module_path
    repo = repo_path
    tech_dir = module_path / "klayout"
    lyp = module_path / "klayout" / "layers.lyp"
    lyp_yaml = module_path / "layers.yaml"
    libs = module_path / "vtt3um"
    sparameters = module_path / "sparameters"

    libs_tech = libs / "libs.tech"
    libs_ref = libs / "libs.ref"
    libs_ngspice = libs_tech / "ngspice"

    gds = module_path / "gds"


PATH = Path()

if __name__ == "__main__":
    print(PATH)



from functools import partial

from gdsfactory.components import mmi2x2 as _mmi2x2

from gvtt.components.transitions import strip_taper
from gvtt.xsections import xs_sc

mmi2x2 = partial(
    _mmi2x2,
    width_mmi=5.0,
    length_mmi=112.0,
    width_taper=1.875,
    cross_section=xs_sc,
    gap_mmi=1.25,
    taper=strip_taper,
    length_taper=1.0,
)


if __name__ == "__main__":
    c = mmi2x2()
    c.show()
